﻿//*****************************************************************************
// Comm: Classe responsável pelas troca de mensagens com os equipamentos Telemed 
//
// Argumentos:
//
// Autor: Gatti - 17/02/2020 - v 2.0.0
//	- Criação do Modulo - Visual Studio 2019
//
// Controle de Alterações:
// <autor> - <data> - v <x.y>
//	- <descrição>
//
// Copyright © Infiza/Telemed 2010-2020
// todos os diretors reservados.
// proibida a utilização/reprodução sem prévia autorização.
//*****************************************************************************
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace Telemed.Communication.Components {

	public enum eType { ELET1 = 0, MT1 = 1, MTV1 = 2 }

	public class Comm {

		private String K_Name = "Comm";
		private String K_Version = "2.0.0";
		private String K_NameVersion = "";

		private eType type = eType.ELET1;
		private string family = "A";
		private int address = 0;
		private int numTanques = 0;
		private string ofe = "";
		private string portName = "";
		private bool isOpen = false;

		private ComIO comIO;

		public string PortName {
			get { return portName; }
		}

		public eType Type {
			get { return type; }
		}

		public string Family {
			get { return family; }
		}

		public int Address {
			get { return address; }
		}

		public int NumeroTanques {
			get { return numTanques; }
		}

		public string OFE {
			get { return ofe; }
		}

		private Comm() { }

		public Comm(eType pType, int pAddress, string pPortName) {
			K_Name = string.Format("{0}.{1}", K_Name, pType.ToString());
			K_NameVersion = K_Name + "-[" + K_Version + "]";

			type = pType;
			address = pAddress;
			portName = pPortName;

			Console.WriteLine(string.Format("{0}.Construct - Port: '{1}', Address: '{2}', Type: '{3}'", K_NameVersion, portName, address, type));

			comIO = new ComIO(portName);
		}

		public void Open() {
			try {
				Console.WriteLine(string.Format("{0}.Open", K_NameVersion));

				try {
					comIO.Open();
				} catch(Exception ex) {
					throw new Exception(string.Format("{0}.Receive: Erro ao abrir porta serial: '{1}'", K_Name, portName), ex);
				}
				try {
					Message mId = new Message("ID", address.ToString("00"));
					comIO.Send(mId.Buf, 1000);
					string m = Receive(1000);

					isOpen = true;
					family = m.Substring(0, 1);
				} catch(Exception ex) {
					throw new Exception(string.Format("{0}.Receive: Erro ao selecionar equipamento: '{1}'", K_Name, address), ex);
				}
				try {
					Thread.Sleep(1000);     // necessario para nao dar timeout no envio (folga para o equipamento)
					string ssssss = "310382";
					Message mCF = new Message("CF", ssssss);
					comIO.Send(mCF.Buf, 1000);
					string m = Receive(1000);
					numTanques = int.Parse(m.Substring(0, 2));
					ofe = m.Substring(2, 6);
				} catch(Exception ex) {
					throw new Exception(string.Format("{0}.Receive: Erro ao ler configuração (número de tanques) do equipamento: '{1}'", K_Name, address), ex);
				}
			} catch(Exception ex) {
				throw new Exception(string.Format("{0}.Receive: Erro ao abrir comunicação com equipamento Telemed: Porta Serial: '{1}', Endereço: '{2}'", K_Name, portName, address), ex);
			}
		}

		public void Close() {
			try {
				comIO.Close();
				isOpen = false;

				Console.WriteLine(K_NameVersion + ".Close");
			} catch(Exception ex) {
				throw new Exception(string.Format("{0}.Close: Erro ao fechar comunicação com equipamento Telemed", K_Name), ex);
			}
		}

		public void Send(string pCommand, string pBody, int pTimeout) {
			Send(pCommand + pBody, pTimeout);
		}

		public void Send(String pInformation, int pTimeout) {
			try {
				if(!isOpen) { Open(); }

				Message m = new Message(pInformation);
				comIO.Send(m.Buf, pTimeout);

				Console.WriteLine(string.Format("{0}.Send: {1}", K_Name, m.ToString()));
			} catch(Exception ex) {
				throw new Exception(string.Format("{0}.Send: Erro ao enviar mensagen", K_Name), ex);
			}
		}

		public string Receive(int pTimeout) {
			try {
				string s = "";

				Message m;
				do {
					byte[] buf = comIO.Receive(pTimeout);
					m = new Message(buf);

					s += m.Body;

					Console.WriteLine(string.Format("{0}.Receive: {1}", K_Name, m.ToString()));
				} while(m.Next);

				return s;
			} catch(Exception ex) {
				throw new Exception(string.Format("{0}.Receive: Erro ao receber mensagen", K_Name), ex);
			}
		}
	}
}
