﻿//*****************************************************************************
// Menssage: Classe responsável pelas mensagens trocadas com os equipamentos
//  Telemed conectados ao varal de comunicação serial 
//
// Argumentos:
//
// Autor: Gatti - 23/11/2010 - v 1.0.0
//  - criação do modulo
//
// Autor: Gatti - 13/01/2012 - v 1.0.1
//  - conversão para Visual Studio 2010
//
// Autor: Gatti - 27/11/2018 - v 1.1.0
//  - Migração para Visual Studio 2017
//
// Autor: Gatti - 17/02/2020 - v 2.0.0
//  - Migração para Visual Studio 2019
//  - Adaptação para nova versão da DLL simples
//
// Controle de Alterações:
// <autor> - <data> - v <x.y>
//  - <descrição>
//
// Copyright © Infiza/Telemed 2010-2020
// todos os diretors reservados.
// proibida a utilização/reprodução sem prévia autorização.
//*****************************************************************************
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Runtime.InteropServices;

namespace Telemed.Communication.Components {

    public class Message {

        private const string K_Name = "Message";
        private const string K_Version = "2.0.0";
        private const string K_NameVersion = K_Name + "[" + K_Version + "]";

        public static byte K_STX = 0x02;        // STX
        public static byte K_SOH = 0x01;        // SOH
        public static byte K_ETX = 0x03;        // ETX
        public static byte K_ACK = 0x06;        // ACK
        public static byte K_NCK = 0x15;        // NCK
        public static byte K_CR = 0x0D;         // CR

        private byte[] buf;
        private byte lrc;
        private int length;

        public byte[] Buf {
            get {
                return buf;
            }
        }

        public int Length {
            get {
                return length;
            }
        }

        public string Information {
            get {
                return Code + Body;
            }
        }
        
        public string Code { 
            get {
                StringBuilder s = new StringBuilder(2);
                s.Append((char)buf[1]);
                s.Append((char)buf[2]);
                return s.ToString(); 
            } 
        }

        public string Body {
            get {
                StringBuilder s = new StringBuilder(length - (1 + 2 + 1 + 1));
                for(int i = 0; i < length - (1 + 2 + 1 + 1); i++) { s.Append((char)buf[i + 3]); }
                return s.ToString();
            }
        }

        public byte LRC {
            get {
                return lrc;
            }
        }
        
        public bool Next {
            get { return Body[Body.Length-1] == K_SOH; }
        }

        public bool Error {
            get { return ((Body.Length < 5) || (Body[3] == 'E')); }
        }
        
        private Message() { }

        public Message(string pCode, string pBody): this(pCode +  pBody) { }
        public Message(string pInformation) {
            length = 1 + pInformation.Length + 1 + 1;   // STX + <Code + Body> + ETX + LRC
            buf = new byte[length];

            buf[0] = K_STX;
            for(int i = 0; i < pInformation.Length; i++) { buf[i + 1] = (byte)pInformation[i]; }
            buf[length - 2] = K_ETX;
            lrc = Calc_LRC();
            buf[length - 1] = lrc;
        }
        public Message(byte[] pBuf) {

            length = pBuf.Length;
            buf = new byte[length];

            for(int i = 0; i < length; i++) { buf[i] = pBuf[i]; }
            lrc = Calc_LRC();
            buf[length - 1] = lrc;
        }

        public override string ToString() {
            return "<STX><" + Code + "><" + Body + "><ETX><0x" + LRC.ToString("X2") + ">";
        }


        private byte Calc_LRC() {
            byte lrc = 0;

            for(int i = 1; i < length - 1; i++) { lrc ^= buf[i]; }

            return lrc;
        }

    }
}
