﻿//*****************************************************************************
// TstTelemedCommunication: Programa de Teste/Exemplo de comunicação com os
//	equipamento Telemed de modelos ELET1, MT1 e MTV1.
//
// Argumentos:
//
// Autor: Gatti - 17/02/2020 - v 2.0.0
//	- Criação do Modulo - Visual Studio 2019
//
// Controle de Alterações:
// <autor> - <data> - v <x.y>
//	- <descrição>
//
// Copyright © Infiza/Telemed 2010-2020
// todos os diretors reservados.
// proibida a utilização/reprodução sem prévia autorização.
//*****************************************************************************
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO.Ports;

using Telemed.Communication.Components;

namespace Telemed.Communication.Test {


	public partial class TstTelemedDLL_Single:Form {

		private string[] K_MsgError = { "Número do tanque inválido", "Tanque não habilitado", "Nâo há medição para a data" };

		private string[] K_DiaSemana = { "Dom", "Seg", "Ter", "Qua", "Qui", "Sex", "Sab" };
		private string[] K_EstadoRbar = { "Ativo", "Inativo" };
		private string[] K_EstadoSensor = { "Cabo em curto-circuito", "Sensor normal", "Vazamento", "Cabo aberto", "Inválido" };

		private string[] K_MsgMt1 = { "IDs", "IDi", "MT", "MA", "AR", "LR", "CF", "LM", "MP" };
		private string[] K_MsgElet1 = { "IDs", "IDi", "MT", "MA", "AR", "LR", "CF", "LM", "MP" };
		private string[] K_MsgMtv1 = { "IDs", "IDi", "MT", "MA", "AR", "LR", "CF", "LM", "MP", "LS" };

		Comm telemed;

		public TstTelemedDLL_Single() {
			InitializeComponent();

			gbCommuniction.Enabled = false;
			cbMessage.Enabled = false;
			btSend.Enabled = false;

			cbCom.DataSource = SerialPort.GetPortNames();
			cbType.DataSource = Enum.GetNames(typeof(eType));
		}

		#region Events
		private void btCreate_Click(object sender, EventArgs e) {
			btCreate.Enabled = false;
			Cursor.Current = Cursors.WaitCursor;
			try {

				if(btCreate.Text == "Open") {
					telemed = new Comm((eType)Enum.Parse(typeof(eType), cbType.Text), 1, cbCom.Text);

					cbMessage.DataSource = ((cbType.SelectedIndex == 0) ? K_MsgElet1 : ((cbType.SelectedIndex == 1) ? K_MsgMt1 : K_MsgMtv1));
					cbCom.Enabled = false;
					cbType.Enabled = false;
					gbCommuniction.Enabled = true;
					btCreate.Text = "Close";
				} else {
					if(telemed != null) { telemed.Close(); }
					telemed = null;

					cbCom.Enabled = true;
					cbType.Enabled = true;
					gbCommuniction.Enabled = false;
					btCreate.Text = "Open";
				}

			} catch(Exception ex) {
				Display(string.Format("{0} Error", btCreate.Text), ex);
			} finally {
				btCreate.Enabled = true;
				Cursor.Current = Cursors.Default;
			}
		}

		private void btConnect_Click(object sender, EventArgs e) {
			btConnect.Enabled = false;
			Cursor.Current = Cursors.WaitCursor;
			try {
				if(btConnect.Text == "Connect") {
					telemed.Open();

					txOfe.Text = telemed.OFE.Trim();
					txTanques.Text = telemed.NumeroTanques.ToString("00");
					if(!(((telemed.Family == "A") && ((cbType.SelectedIndex == 0) || (cbType.SelectedIndex == 1))) || ((telemed.Family == "B") && (cbType.SelectedIndex == 2)))){
						Display(string.Format("A Familia de equipamento conectado não confere com o escolhido !! Equipamento: {0} - Escolhido: {1}", Familia(telemed.Family), cbType.Text));
					}

					btCreate.Enabled = false;
					cbMessage.Enabled = true;
					btSend.Enabled = true;
					btConnect.Text = "Disconnect";
				} else {
					telemed.Close();

					txOfe.Text = "";
					txTanques.Text = "";

					btCreate.Enabled = true;
					cbMessage.Enabled = false;
					btSend.Enabled = false;
					btConnect.Text = "Connect";
				}
			} catch(Exception ex) {
				Display(string.Format("{0} Error", btConnect.Text), ex);
			} finally {
				btConnect.Enabled = true;
				Cursor.Current = Cursors.Default;
			}
		}

		private void btSend_Click(object sender, EventArgs e) {
			btConnect.Enabled = false;
			btSend.Enabled = false;
			Cursor.Current = Cursors.WaitCursor;
			try {
				switch(cbMessage.SelectedValue) {
					case "IDs": MsgIDSelect(); break;
					case "IDi": MsgIDIdentification(); break;
					case "MT": MsgMT(); break;
					case "MA": MsgMA(); break;
					case "AR": MsgAR(); break;
					case "LR": MsgLR(); break;
					case "CF": MsgCF(); break;
					case "LM": MsgLM(); break;
					case "MP": MsgMP(); break;
					case "LS": MsgLS(); break;
				}

			} catch(Exception ex) {
				Display("Send/Receive Error", ex);
			} finally {
				btConnect.Enabled = true;
				btSend.Enabled = true;
				Cursor.Current = Cursors.Default;
			}
		}

		private void btClean_Click(object sender, EventArgs e) {
			txMessage.Text = "";
		}

		private void btExit_Click(object sender, EventArgs e) {
			Close();
		}
		#endregion Events

		#region Display
		private void Display(string pMessage) {
			txMessage.AppendText(pMessage + Environment.NewLine);
			txMessage.ScrollToCaret();
		}

		private void Display(string pMessage, Exception pEx) {
			string msg = pMessage + Environment.NewLine;

			Exception ex = pEx;
			while(ex != null) {
				msg += "  - " + ex.Message + Environment.NewLine;
				ex = ex.InnerException;
			}

			Display(msg);
		}
		#endregion Display

		#region General
		private string Familia(string pFamilia) {
			// Retorna a familia do equipamento
			return pFamilia[0] == 'A' ? "Familia MT" : "Familia MV";
		}

		private string Hora(string pHora) {
			// Formata Hora
			return pHora.Substring(0, 2) + ":" + pHora.Substring(2, 2) + ":" + pHora.Substring(4, 2);
		}

		private string Data(string pData) {
			// Formata Data
			return pData.Substring(0, 2) + "/" + pData.Substring(2, 2) + "/" + pData.Substring(4, 2);
		}

		private string Unidade(string pUnidade) {
			// Retorna a unidade de medida
			return (pUnidade[0] == 'P' ? "Kilo" : "Litros");
		}

		private string DiaSemana(string pDiaSemana) {
			// Retorna o dia da semana
			return K_DiaSemana[int.Parse(pDiaSemana.Substring(0, 1))];
		}

		private string TipoMedidor(string pTipoMedidor) {
			// Retorna o tipo do medidor
			return pTipoMedidor[0] == '0' ? "Industrial" : "Posto";
		}

		private string EstadoSensor(string pEstadoSensor) {
			// Retorna o estado de um sensor
			int index;
			if(pEstadoSensor == "1") {
				index = 0;
			} else if(pEstadoSensor == "2") {
				index = 1;
			} else if(pEstadoSensor == "3") {
				index = 2;
			} else if(pEstadoSensor == "4") {
				index = 3;
			} else {
				index = 4;
			}
			return K_EstadoSensor[index];
		}

		private string EstadoRbar(string pEstaoRbar) {
			// Retorna o estado de uma RBAR
			return pEstaoRbar == "A" ? K_EstadoRbar[0] : K_EstadoRbar[1];
		}
		#endregion General

		#region Message Format
		private string FormatsLeituraTanqueMt1(string pInformacao) {
			// Formata a mensagem de retorno da Medição de tanque para um tanque

			//“NNNDDDDPPCCCCCCVVVVVVMMMMMMHHMMSSDDMMAAUT” 
			// 0        1         2         3         4 
			// 12345678901234567890123456789012345678901
			// NNN - Número do tanque
			// DDDD - Densidade do produto
			// PP - Identificação do produto
			// CCCCCC - Capacidade do Tanque
			// VVVVVV - Volume Medido
			// MMMMMM - Volume Complementar
			// HHMMSS - Hora da leitura
			// DDMMAA - Data da leitura
			// U - Unidade de apresentação - P=Quilogramas, V=Litros
			// T = Teminador - <SOH>-Tem mais mensagem, <CR>-Ultima mensagem

			string s = "";

			byte terminator = 0;
			int i = 0;
			do {
				if(((pInformacao.Length - i) <= 3) || (pInformacao[i] == 'E')) {     // formato do corpo da memsagem de Erro = 'ETT'
					s += "rx: " + MsgError(pInformacao, i); i += (i + 3) >= pInformacao.Length ? 2 : 3;
				} else {
					s += "rx:" + Environment.NewLine;
					s += "    Número do Tanque.....: " + pInformacao.Substring(i, 3) + Environment.NewLine; i += 3;
					s += "    Densidade do Produto.: " + pInformacao.Substring(i, 4) + Environment.NewLine; i += 4;
					s += "    Identificação Produto: " + pInformacao.Substring(i, 2) + Environment.NewLine; i += 2;
					s += "    Capacidade do Tanque.: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
					s += "    Volume Medido........: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
					s += "    Volume Complementar..: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
					s += "    Hora da Leitura......: " + Hora(pInformacao.Substring(i, 6)) + Environment.NewLine; i += 6;
					s += "    Data da Leitura......: " + Data(pInformacao.Substring(i, 6)) + Environment.NewLine; i += 6;
					s += "    Unidade apresentação.: " + Unidade(pInformacao.Substring(i, 1)) + Environment.NewLine; i += 1;
				}

				terminator = (byte)pInformacao[i]; i += 1;
			} while(terminator == Telemed.Communication.Components.Message.K_SOH);

			return s;
		}

		private string FormatsLeituraTanqueMtv1(string pInformacao) {
			// Formata a mensagem de retorno da Medição de tanque para um tanque

			//“MTNNNPPPPPPPPPPPFFFFFFDDDDCCCCCCVVVVVVMMMMMMXXXXXXRRRRRRHHMMSSDDMMAAU” 
			// 0        1         2         3         4 
			// 12345678901234567890123456789012345678901
			// NNN - Número do tanque
			// PPPPPPPPPPP - Identificação do produto
			// FFFFFF - Coeficiente de Expansão
			// DDDD - Densidade do produto
			// CCCCCC - Capacidade do Tanque
			// VVVVVV - Volume Medido
			// MMMMMM - Volume Complementar
			// XXXXXX - Volume Medido a 20oC
			// RRRRRR - Altura do produto no tanque
			// HHMMSS - Hora da leitura
			// DDMMAA - Data da leitura
			// U - Unidade de apresentação - P=Quilogramas, V=Litros
			// T = Teminador - <SOH>-Tem mais mensagem, <CR>-Ultima mensagem

			string s = "";
			byte terminator = 0;
			int i = 0;
			do {
				if(((pInformacao.Length - i) <= 3) || (pInformacao[i] == 'E')) {     // formato do corpo da memsagem de Erro = 'ETT'
					s += "rx: " + MsgError(pInformacao, i); i += (i + 3) >= pInformacao.Length ? 2 : 3;
				} else {
					s += "rx:" + Environment.NewLine;
					s += "    Número do Tanque................: " + pInformacao.Substring(i, 3) + Environment.NewLine; i += 3;
					s += "    Identificação Produto...........: " + pInformacao.Substring(i, 11) + Environment.NewLine; i += 11;
					s += "    Coeficiente de Expansão Termica.: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
					s += "    Densidade do Produto............: " + pInformacao.Substring(i, 4) + Environment.NewLine; i += 4;
					s += "    Capacidade do Tanque............: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
					s += "    Volume Medido...................: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
					s += "    Volume Complementar.............: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
					s += "    Volume medido a 20oC............: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
					s += "    Altura do produto no tanque.....: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
					s += "    Hora da Leitura.................: " + Hora(pInformacao.Substring(i, 6)) + Environment.NewLine; i += 6;
					s += "    Data da Leitura.................: " + Data(pInformacao.Substring(i, 6)) + Environment.NewLine; i += 6;
					s += "    Unidade apresentação............: " + Unidade(pInformacao.Substring(i, 1)) + Environment.NewLine; i += 1;
				}

				terminator = (byte)pInformacao[i]; i += 1;
			} while(terminator == Telemed.Communication.Components.Message.K_SOH);

			return s;
		}

		private string FormataLeituraProgramada(string pInformacao) {
			// Formata a mensagem de retorno da Medição Programada para um tanque

			//“HHMMSSDDMMAAVVVVVVNNNT” 
			// 0        1         2  
			// 1234567890123456789012
			// HHMMSS - Hora da medição
			// DDMMAA - Data da medição
			// VVVVVV - Vaoume em litros
			// NNN - Número do tanque medido
			// T - Teminador - <SOH>=Tem mais mensagem, <CR>=Ultima mensagem

			string s = "";
			byte terminator = 0;
			int i = 0;
			do {
				if(((pInformacao.Length - i) <= 3) || (pInformacao[i] == 'E')) {     // formato do corpo da memsagem de Erro = 'ETT'
					s += "rx: " + MsgError(pInformacao, i);  i += (i+3)>=pInformacao.Length ? 2 : 3;
				} else {
					s += "rx:" + Environment.NewLine;
					s += "    Hora da Medição..: " + Hora(pInformacao.Substring(i, 6)) + Environment.NewLine; i += 6;
					s += "    Data da Medição..: " + Data(pInformacao.Substring(i, 6)) + Environment.NewLine; i += 6;
					s += "    Volume Medido....: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
					s += "    Número do Tanque.: " + pInformacao.Substring(i, 3) + Environment.NewLine; i += 3;
				}

				terminator = (byte)pInformacao[i]; i += 1;
			} while(terminator == Telemed.Communication.Components.Message.K_SOH);

			return s;
		}

		private string FormataLeituraRelogio(string pInformacao) {
			// Formata a mensagem de retorno para a Alteração e Leitur do Relógio

			//“HHMMSSDDMMAADx”
			// 0        1    
			// 12345678901234
			// HHMMSS - Hora atual do relógio
			// DDMMAA - Data atual do relógio
			// D - Dia da semana - 1 a 7, 1=Domingo
			// x - desprezar

			string s = "";
			int i = 0;
			if(((pInformacao.Length - i) <= 3) || (pInformacao[i] == 'E')) {     // formato do corpo da memsagem de Erro = 'ETT'
				s += "rx: " + MsgError(pInformacao, i); i += (i + 3) >= pInformacao.Length ? 2 : 3;
			} else {
				s += "rx:" + Environment.NewLine;
				s += "    Hora Atual.....: " + Hora(pInformacao.Substring(i, 6)) + Environment.NewLine; i += 6;
				s += "    Data Atual.....: " + Data(pInformacao.Substring(i, 6)) + Environment.NewLine; i += 6;
				s += "    Dia da Semana..: " + DiaSemana(pInformacao.Substring(i, 1)) + Environment.NewLine; i += 1;
			}

			return s;
		}

		private string FormataLeituraConfiguracao(string pInformacao) {
			// Formata a mensagem de retorno para a Leitura da Configuração

			//“NTXXXXXXM” 
			// 0         
			// 123456789
			// NT - Nùmero de tanques
			// XXXXXX - Número da OFE (Ordem de Fabricção de Equipamento)
			// M - Tipo de Medidor - 1=Posto, 0=Industial

			string s = "";
			int i = 0;
			if(((pInformacao.Length - i) <= 3) || (pInformacao[i] == 'E')) {     // formato do corpo da memsagem de Erro = 'ETT'
				s += "rx: " + MsgError(pInformacao, i); i += (i + 3) >= pInformacao.Length ? 2 : 3;
			} else {
				s += "rx:" + Environment.NewLine;
				s += "    Número de Tanque.: " + pInformacao.Substring(i, 2) + Environment.NewLine; i += 2;
				s += "    Número da OFE....: " + pInformacao.Substring(i, 6) + Environment.NewLine; i += 6;
				s += "    Tipo de Medidor..: " + TipoMedidor(pInformacao.Substring(i, 1)) + Environment.NewLine; i += 1;
			}

			return s;
		}

		private string FormataLeituraIdentificacao(string pInformacao) {
			// Formata a mensagem de retorno para a Seleção/Identificação do equipamento selecionado

			//“XNNVV”
			// 12345
			// X - Identificador do sistema - A=familia MT B=Familia MV
			// NN - Endereço de comunicação MT-00 a 01, MV-00 a 32
			// VV - Verão de comunicação do sistema - 01

			string s = "";
			int i = 0;
			if(((pInformacao.Length - i) <= 3) || (pInformacao[i] == 'E')) {     // formato do corpo da memsagem de Erro = 'ETT'
				s += "rx: " + MsgError(pInformacao, i); i += (i + 3) >= pInformacao.Length ? 2 : 3;
			} else {
				s += "rx:" + Environment.NewLine;
				s += "    Sistema.: " + Familia(pInformacao.Substring(i, 1)) + Environment.NewLine; i += 1;
				s += "    Endereço: " + pInformacao.Substring(i, 2) + Environment.NewLine; i += 2;
				s += "    Versão..: " + pInformacao.Substring(i, 2) + Environment.NewLine; i += 2;
			}

			return s;
		}

		private string FormataLeituraSensor(string pInformacao) {
			// Formata a mensagem de retorno para a Leitura dos Sensores

			//“XXXXXXXXXXXXXXXXEXXXXXXXXXXXXXXXXE
			// 0         1        2         3  
			// 1234567890123456789012345678901234
			// E - Estado da cada RBAR (no máximo 2 RBAR) - A=Ativo, I=Inativo
			// X - Estado do Sensor - 1=Cabo em curto-circuito, 2=Sensor normal, 3=vazamento, 4=Cabo aberto

			string s = "";
			int i = 0;
			if(((pInformacao.Length - i) <= 3) || (pInformacao[i] == 'E')) {     // formato do corpo da memsagem de Erro = 'ETT'
				s += "rx: " + MsgError(pInformacao, i); i += (i + 3) >= pInformacao.Length ? 2 : 3;
			} else {
				s += "rx:" + Environment.NewLine;
				s += "RBAR-1: " + EstadoRbar(pInformacao.Substring(16, 1)) + Environment.NewLine;
				for(int index = 0; index < 16; index++) {
					s += string.Format("Sensor[{0}]: {1}", index.ToString("00"), EstadoSensor(pInformacao.Substring(index, 1))) + Environment.NewLine;
				}
				s += "RBAR-2: " + EstadoRbar(pInformacao.Substring(33, 1)) + Environment.NewLine;
				for(int index = 17; index < 33; index++) {
					s += string.Format("Sensor[{0}]: {1}", (index - 1).ToString("00"), EstadoSensor(pInformacao.Substring(index, 1))) + Environment.NewLine;
				}
			}

			return s;
		}
		#endregion Message Format

		#region Menssage
		private void MsgIDSelect() {
			//“IDNN”  - Seleciona um equipamento
			// 1234
			// ID = Código do comando
			// NN = endereço do equipamento - 01 a 16 - famila MT e 01 a 32 - famila MV

			string end = "01";
			string msg = "ID" + end;
			try {
				Display("tx: " + msg);

				telemed.Send(msg, 1000);
				string resp = telemed.Receive(1000);

				Display(FormataLeituraIdentificacao(resp));
			} catch(Exception ex) {
				Display("IDs Error", ex);
			}
		}

		private void MsgIDIdentification() {
			//“ID” - Retorna a identificação do equipamento selecionado
			// ID = Código do comando

			string msg = "ID";
			try {
				Display("tx: " + msg);

				telemed.Send(msg, 1000);
				string resp = telemed.Receive(1000);

				Display(FormataLeituraIdentificacao(resp));
			} catch(Exception ex) {
				Display("IDi Error", ex);
			}
		}

		private void MsgMT() {
			//“MTNN” - Medição de Tanque
			// 1234
			// MT - Código do comando
			// NN - Número do Tanque - 01 a 16

			string nn = "01";
			string msg = "MT" + nn;
			try {
				Display("tx: " + msg);

				telemed.Send(msg, 1000);
				string resp = telemed.Receive(30000);

				if(telemed.Type == eType.MTV1) {
					Display(FormatsLeituraTanqueMtv1(resp));
				} else {
					Display(FormatsLeituraTanqueMt1(resp));
				}
			} catch(Exception ex) {
				Display("MT Error", ex);
			}
		}

		private void MsgMA() {
			//"MA" - Medição de todos os tanques
			// 12
			// MA - Código do comando

			string msg = "MA";
			try {
				Display("tx: " + msg);

				telemed.Send(msg, 1000);
				string resp = telemed.Receive(30000);

				if(telemed.Type == eType.MTV1) {
					Display(FormatsLeituraTanqueMtv1(resp));
				} else {
					Display(FormatsLeituraTanqueMt1(resp));
				}
			} catch(Exception ex) {
				Display("MA Error", ex);
			}
		}

		private void MsgAR() {
			//“ARSSSSSSMHHMMSSDDMMAA” - Alteração do Relógio
			// 0        1         2
			// 123456789012345678901
			// AR - Código do comando
			// SSSSSS - Senha para acesso ao equipamento
			// M - Dia da Semana - 1 a 7, sendo 1=domingo
			// HHMMSS - Novo Horário mm-hora, mm-minuto, ss-segundo
			// DDMMAA - Nova Data dd-dia, mm-mês, aa-ano

			string ssssss = "310382";
			string m = (DateTime.Today.DayOfWeek).ToString("d");
			string hhmmss = (DateTime.Now.ToString("HHmmss"));
			string ddmmaa = (DateTime.Today.ToString("ddMMyy"));
			string msg = "AR" + ssssss + m + hhmmss + ddmmaa;
			try {
				Display("tx: " + "AR" + "******" + m + hhmmss + ddmmaa);

				telemed.Send(msg, 1000);
				string resp = telemed.Receive(2000);

				Display(FormataLeituraRelogio(resp));
			} catch(Exception ex) {
				Display("AR Error", ex);
			}
		}

		private void MsgLR() {
			//"LR" - Leitura do Relógio
			// 12
			// AR - Código do comando

			string msg = "LR";
			try {
				Display("tx: " + msg);

				telemed.Send(msg, 1000);
				string resp = telemed.Receive(2000);

				Display(FormataLeituraRelogio(resp));
			} catch(Exception ex) {
				Display("LR Error", ex);
			}
		}

		private void MsgCF() {
			//“CFSSSSSS” - Leitura da Configuração
			// 12345678
			// CF - Código do comando
			// SSSSSS - Senha para acesso ao equipamento

			string ssssss = "310382";
			string msg = "CF" + ssssss;
			try {
				Display("tx: " + "CF" + "******");

				telemed.Send(msg, 1000);
				string resp = telemed.Receive(2000);

				Display(FormataLeituraConfiguracao(resp));
			} catch(Exception ex) {
				Display("CF Error", ex);
			}
		}

		private void MsgLM() {
			//“LMNN” - Leitura da Ultima medição do tanque
			// 1234
			// LM - Código do comando
			// NN - Número do Tanque

			string nn = "01";
			string msg = "LM" + nn;
			try {
				Display("tx: " + msg);

				telemed.Send(msg, 1000);
				string resp = telemed.Receive(2000);

				if(telemed.Type == eType.MTV1) {
					Display(FormatsLeituraTanqueMtv1(resp));
				} else {
					Display(FormatsLeituraTanqueMt1(resp));
				}
			} catch(Exception ex) {
				Display("LM Error", ex);
			}
		}

		private void MsgMP() {
			//“MPDDMMAA” - Leitura das Medições Programadas
			// 12345678
			// MP - Código do comando
			// DDMMAA - Data da Leitura Programada - dd-dia, mm-mês, aa-ano

			string ddmmaa = (DateTime.Today.ToString("ddMMyy"));
			string msg = "MP" + ddmmaa;
			try {
				Display("tx: " + msg);

				telemed.Send(msg, 1000);
				string resp = telemed.Receive(20000);

				Display(FormataLeituraProgramada(resp));
			} catch(Exception ex) {
				Display("MP Error", ex);
			}
		}

		private void MsgLS() {
			//"LS" - Leitura dos Sensores de Vazamento
			// 12
			// LS - Código do comando

			string msg = "LS";
			try {
				Display("tx: " + msg);

				telemed.Send(msg, 1000);
				string resp = telemed.Receive(20000);

				Display(FormataLeituraSensor(resp));
			} catch(Exception ex) {
				Display("LS Error", ex);
			}
		}

		private string MsgError(string pResp, int pIndex) {
			//“ETT”
			// 012

			string s = "";
			string m = "";
			if((pIndex + 3) > pResp.Length) {
				s = "Mensage de erro inválida";
				m = pResp;
			} else {
				m = pResp.Substring(pIndex, 3);
				string codeError = pResp.Substring(pIndex + 1, 2);
				if(codeError == "TI") {
					s = K_MsgError[0];
				} else if(codeError == "NH") {
					s = K_MsgError[1];
				} else if(codeError == "SL") {
					s = K_MsgError[2];
				} else {
					s = "Código de erro inválido";
				}
			}
			return string.Format("'{0}' - {1}\r\n", m, s);
		}
		#endregion Message
	}
}
